--[[
Thanks to:
Snorgrind MacLib addon for target buff/debuff discovery method (tooltip text)
Zoltanus & Voltan of Cure Gaming for DE localizations
Feu of Curse Gaming for FR localizations

]]--

local SEAL_JUDGE_VERSION = "0.6.3";

-- myAddons Details
SealJudgeDetails = {
  name = 'SealJudge',
  version = SEAL_JUDGE_VERSION,
  releaseDate = 'Sep 19, 2006',
  author = 'Arkaria',
  email = 'arkaria@arkaria.org',
  website = 'http://ui.worldofwar.net/ui.php?id=1858',
  category = MYADDONS_CATEGORY_COMBAT,
  optionsframe = 'SealJudgeOptionsFrame'
};

-- myAddons Help Pages
SealJudgeHelp = {};
SealJudgeHelp[1] = "/sj cycle - Cycle through Seal/Judgement sequence.\n'/sj config' - Show options frame.";

function SealJudge_OnLoad()
  SlashCmdList["SEALJUDGESLASH"] = SealJudge_Main_ChatCommandHandler;
  SLASH_SEALJUDGESLASH1 = "/sj";
  SLASH_SEALJUDGESLASH2 = "/sealjudge";

  -- Register the events that need to be watched for
  this:RegisterEvent("ADDON_LOADED");
  this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("PLAYER_ENTER_COMBAT");
  this:RegisterEvent("PLAYER_LEAVE_COMBAT");
  this:RegisterEvent("SPELLCAST_START");
  this:RegisterEvent("SPELLCAST_FAILED");
  this:RegisterEvent("SPELLCAST_STOP");
  this:RegisterEvent("SPELLCAST_INTERRUPTED");
end;

function SealJudge_OnEvent()
  if event == "ADDON_LOADED" and arg1 == "SealJudge" then
    -- Check if myAddOns is loaded
    if(myAddOnsFrame_Register) then
      -- Register the addon in myAddOns
      myAddOnsFrame_Register(SealJudgeDetails, SealJudgeHelp);
    end
  end
  if event == "VARIABLES_LOADED" then
    SealJudge_LoadVariables();
  end;
  if event == "PLAYER_ENTER_COMBAT" then
    SealJudge_Status.inCombat = true;
  elseif event == "PLAYER_LEAVE_COMBAT" then
    SealJudge_Status.inCombat = false;
  end;
  if event == "SPELLCAST_START" then
    SealJudge_Status.Casting = true;
  elseif event == "SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED" or event == "SPELLCAST_STOP" then
    SealJudge_Status.Casting = false;
  end;
end;

function SealJudge_LoadVariables()
  local info = UnitName("player").." of "..GetCVar("realmName"); 
  if not SealJudgeConfig then
    SealJudgeConfig = {};
    SealJudge_Status.inCombat = false;
    SealJudge_Status.Casting = false;
  end;
  if not SealJudge_Status then
    SealJudge_Status = {};
  end;
  -- Set Character Defaults
  if not SealJudgeConfig[info] then
    SealJudgeConfig[info] = {
      ["version"] = SEAL_JUDGE_VERSION,
      ["Enabled"] = true,
      ["JudgeSeal"] = true,
      ["JudgeSeal_ToBeJudged"] = SEALJUDGE_JUDGESEAL_CRUSADER,
      ["CombatSeal"] = true,
      ["CombatSeal_ToKeepUp"] = SEALJUDGE_COMBATSEAL_COMMAND,
      ["CombatSealAlways"] = false,
      ["JudgeSealWhen"] = "Stun",
      ["ExorcismWhenReady"] = false,
      ["HammerOfWrathWhenReady"] = false,
      ["HolyShockWhenReady"] = false
    };
  elseif not SealJudgeConfig[info].version then
    SealJudgeConfig[info].JudgeSealWhen = "Stun";
    SealJudgeConfig[info].JudgeCommandOnStun = nil;
    SealJudgeConfig[info].JudgeSeal_ToBeJudged = SEALJUDGE_JUDGESEAL_CRUSADER;
    SealJudgeConfig[info].CombatSeal_ToKeepUp = SEALJUDGE_COMBATSEAL_COMMAND;
  elseif SealJudgeConfig[info].version == "0.3" or SealJudgeConfig[info].version == "0.3.1" then
    SealJudgeConfig[info].JudgeSeal_ToBeJudged = SEALJUDGE_JUDGESEAL_CRUSADER;
    SealJudgeConfig[info].CombatSeal_ToKeepUp = SEALJUDGE_COMBATSEAL_COMMAND;
  end;
  if SealJudgeConfig[info].JudgeSealOnStun then
    SealJudgeConfig[info].JudgeSealOnStun = nil;
    SealJudgeConfig[info].JudgeSealWhen = "Stun";
  end;
  if not SealJudgeConfig[info].ExorcismWhenReady then
    SealJudgeConfig[info].ExorcismWhenReady = false;
  end;
  if not SealJudgeConfig[info].HammerOfWrathWhenReady then
    SealJudgeConfig[info].HammerOfWrathWhenReady = false;
  end;
  if not SealJudgeConfig[info].HolyShockWhenReady then
    SealJudgeConfig[info].HolyShockWhenReady = false;
  end;
  if not SealJudgeConfig[info].CombatSealAlways then
    SealJudgeConfig[info].CombatSealAlways = false;
  end;
  SealJudgeConfig[info].version = SEAL_JUDGE_VERSION;
  SJConfig = SealJudgeConfig[info];
  DEFAULT_CHAT_FRAME:AddMessage("SealJudge "..SEAL_JUDGE_VERSION.." Loaded", 1, 1, 0);
end;

function SealJudge_Main_ChatCommandHandler(msg)
  local msg = TextParse(msg);
  msg[1] = strlower(msg[1])
  if strfind(msg[1], "cycle") then
    SealJudge_DoCycle();
  elseif strfind(msg[1], "options") or strfind(msg[1], "config") then
    SealJudgeOptionsFrame:Show();
  else
    DEFAULT_CHAT_FRAME:AddMessage(SEALJUDGE_COMMANDS, 1, 1, 0);
  end;
end;



-- Check to see if the current target is dead
function isTargetDead()
  if UnitIsDeadOrGhost("target") then
    debug_me("Unit is Dead or a Ghost");
    return true;
  end;
  if UnitIsCorpse("target") then
    debug_me("Unit is a Corpse");
    return true;
  end;
  debug_me("Unit is Not Dead");
  return false;
end;

-- Check to see if the current target is valid for attack
function SJisValidTarget()
  if (UnitExists("target")) then
    if (UnitReaction("player", "target") > 4) then
      debug_me("This is a Friendly Target");
      return false;
    elseif (not UnitCanAttack("player", "target")) then
      debug_me("Can not attack Unit");
      return false;
    elseif (isTargetDead()) then
      return false;
    elseif (UnitIsCharmed("target")) then
      debug_me("Unit is Charmed");
      return false;
    else
      debug_me("Target is Valid");
      return true;
    end;
  elseif (SJConfig.CombatSealAlways == true) then
    debug_me("Invalid Target, Always keep comabt Seal enabled... Checking");
    checkCombatSeal();
    return false;
  else
    debug_me("No target selected.");
    return false;
  end;
end;

-- Checks if the seal to be judged has been judged and if not judge it
function checkSealToBeJudged()
  if (SJConfig.JudgeSeal == false) then
    return true;
  end
  local JudgeID = GetSpellID(SEALJUDGE_JUDGESEAL_JUDGEMENT);
  if (sj_getDebuffIndex(SEALJUDGE_JUDGESEAL_JUDGEMENT.." "..SJConfig.JudgeSeal_ToBeJudged) == 0) then
    debug_me("Unit is Not Judged");
    if (sj_getBuffIndex(SEALJUDGE_JUDGESEAL_SEAL.." "..SJConfig.JudgeSeal_ToBeJudged) == 0 and CheckCooldown(JudgeID)) then
      debug_me("Seal to be Judged is not Up and Judgement is ready... Casting");
      CastSpellByName(SEALJUDGE_JUDGESEAL_SEAL.." "..SJConfig.JudgeSeal_ToBeJudged);
      SpellStopCasting();
      return false
    elseif (CheckCooldown(JudgeID)) then
      debug_me("Judging Unit");
      CastSpellByName(SEALJUDGE_JUDGESEAL_JUDGEMENT);
      SpellStopCasting();
      checkCombatSeal();
    else
      debug_me("Judgement Not Ready");
      return false
    end;
  else 
    debug_me("Unit is Judged");
    return true;
  end;
end;

-- Checks if the combat seal is up if not cast it
function checkCombatSeal()
  if (SJConfig.CombatSeal == false) then
    return true;
  end;
  local SealID = GetSpellID(SJConfig.CombatSeal_ToKeepUp);
  if (CheckCooldown(SealID) and sj_getBuffIndex(SJConfig.CombatSeal_ToKeepUp) == 0) then
    debug_me("Combat Seal is not Up and is ready to be cast... Casting");
    CastSpellByName(SJConfig.CombatSeal_ToKeepUp);
    SpellStopCasting();
  else 
    debug_me("Combat Seal is Up");
    return true;
  end;
end;

-- Check if the unit is stunned or disoriented
function isUnitStunned(thisTarget)
  local retValue = false;
  local tooltip = SJTooltip;
  local toolTipLineText;
  for i = 1,16,1 do
    tooltip:ClearLines();
    tooltip:SetUnitDebuff(thisTarget, i);
    local toolTipLines = tooltip:NumLines();
    for j = 1,toolTipLines,1 do
      toolTipLineText = getglobal("SJTooltipTextLeft"..j);
      toolTipLineText = toolTipLineText:GetText();
      if (string.find(toolTipLineText, TARGET_STUNNED) ~= nil or string.find(toolTipLineText, TARGET_DISORIENTED) ~= nil or string.find(toolTipLineText, TARGET_INCAPACITATED) ~= nil) then
        debug_me("Unit is Stunned/Disoriented/Incapacitated");
        retValue = true;
        break;
      end;
    end;
  end;
  return retValue;
end;

function checkStun()
  if (SJConfig.JudgeSealWhen == "Stun" and isUnitStunned("target")) then
    local JudgeID = GetSpellID(SEALJUDGE_JUDGESEAL_JUDGEMENT);
    if (checkCombatSeal() and CheckCooldown(JudgeID)) then
      debug_me("Judging on Stun");
      CastSpellByName(SEALJUDGE_JUDGESEAL_JUDGEMENT);
      SpellStopCasting();
      checkCombatSeal();
    end;
  end;
end;

function checkJudgment()
  local JudgeID = GetSpellID(SEALJUDGE_JUDGESEAL_JUDGEMENT);
  if (SJConfig.JudgeSealWhen == "Always" and CheckCooldown(JudgeID)) then
    debug_me("Judgment is ready... Casting");
    CastSpellByName(SEALJUDGE_JUDGESEAL_JUDGEMENT);
    SpellStopCasting();
    checkCombatSeal();
  else
    debug_me("Judgment is Not ready or Judge when ready is not set.");
  end;
end;

function checkExorcism()
  if (SJConfig.ExorcismWhenReady) then
    local ExorcismID = GetSpellID(COOLDOWN_CHECK_EXORCISM);
    if (CheckCooldown(ExorcismID)) then
      debug_me("Exorcism is ready");
      if (isTargetType(TARGET_TYPE_UNDEAD) or isTargetType(TARGET_TYPE_DEMON)) then
        debug_me("Target is Undead/Demon... Casting");
        CastSpellByName(COOLDOWN_CHECK_EXORCISM);
        SpellStopCasting();
      else
        debug_me("Target is NOT Undead/Demon");
      end;
    else
      debug_me("Exorcism is NOT ready");
    end;
  end;
end;

function checkHammerOfWrath()
  if (SJConfig.HammerOfWrathWhenReady) then
    local HammerID = GetSpellID(COOLDOWN_CHECK_HAMMEROFWRATH);
    if (CheckCooldown(HammerID)) then
      debug_me("Hammer Of Wrath is ready");
      if (UnitHealth("target") < 20) then
        debug_me("Target is Below 20% Life... Casting");
        CastSpellByName(COOLDOWN_CHECK_HAMMEROFWRATH);
      else
        debug_me("Target is NOT Below 20% Life");
      end;
    else
      debug_me("Hammer Of Wrath is NOT ready");
    end;
  end;
end;

function checkHolyShock()
  if (SJConfig.HolyShockWhenReady) then
    local ShockID = GetSpellID(COOLDOWN_CHECK_HOLYSHOCK);
    if (CheckCooldown(ShockID)) then
      debug_me("Holy Shock is ready");
      CastSpellByName(COOLDOWN_CHECK_HOLYSHOCK);
      SpellStopCasting();
    else
      debug_me("Holy Shock is NOT ready");
    end;
  end;
end;

function SealJudge_DoCycle()
  if (SJConfig.Enabled == true) then
    debug_me("\n<------Cycle------>");
    if (SJisValidTarget()) then
      if (not SealJudge_Status.inCombat) then
        debug_me("Auto Attack On");
        AttackTarget();
      end;
      checkStun();
      if (checkSealToBeJudged()) then
        if (checkCombatSeal()) then
          checkStun();
          checkJudgment();
          checkExorcism();
          checkHammerOfWrath();
          checkHolyShock();
        end
      end
    end
  end
end
















-- Debug message
function debug_me(my_message)
  if (DEBUG_ME) then
    DEFAULT_CHAT_FRAME:AddMessage(my_message, 1, 0, 0);
  end;
end;

-- [U] Get the index of the specified spellname.  Returns 0 if no such buff.
function sj_getBuffIndex(spellname, unit)
  if (unit == nil) then unit = "player"; end;
  local iBuff = 0;

  for i = 1,16,1 do
    local buffspell = sj_spellForBuff(unit, i);
    if(buffspell == spellname) then
      iBuff = i;
      break;
    end;
  end
  
  return iBuff;
end

-- [U] Get the index of the specified spellname.  Returns 0 if no such debuff.
function sj_getDebuffIndex(spellname, unit)
  if (unit == nil) then unit = "target"; end;
  local iBuff = 0;

  for i = 1,16,1 do
    local buffspell = sj_spellForDebuff(unit, i);
    if(buffspell == spellname) then
      iBuff = i;
      break;
    end;
  end;
  return iBuff;
end

-- given a unit and buff index, return the name of the spell (as seen in tooltip)
function sj_spellForBuff(unit, iBuff)
  if (iBuff >= 1) and (iBuff < 17) then
    local tooltip = SJTooltip;
    local name = nil;
    tooltip:ClearLines();
    tooltip:SetUnitBuff(unit, iBuff);
    local toolTipText = getglobal("SJTooltipTextLeft1");
    if (toolTipText) then
      name = toolTipText:GetText();
      if (name ~= nil) then
        return name;
      end;
    end;
  end;
end;

-- given a unit and debuff index, return the name of the spell (as seen in tooltip)
function sj_spellForDebuff(unit, iBuff)
  if (iBuff >= 0) and (iBuff < 17) then
    local tooltip = SJTooltip;
    tooltip:ClearLines();
    tooltip:SetUnitDebuff(unit, iBuff);
    local toolTipText = getglobal("SJTooltipTextLeft1");
    if (toolTipText) then
      local name = toolTipText:GetText();
      if (name ~= nil) then
        return name;
      end;
    end;
  end;
end;

function isTargetType(typeText)
  local tooltip = SJTooltip;
  local retValue = false;
  tooltip:ClearLines();
  tooltip:SetUnit("target");
  local toolTipText = getglobal("SJTooltipTextLeft1");
  if (toolTipText) then
    local toolTipLines = tooltip:NumLines();
    for j = 1,toolTipLines,1 do
      toolTipLineText = getglobal("SJTooltipTextLeft"..j);
      toolTipLineText = toolTipLineText:GetText();
      if (string.find(toolTipLineText, typeText) ~= nil) then
        debug_me("Unit is "..typeText);
        retValue = true;
        break;
      end;
    end;
  end;
  return retValue;
end;

-- Returns the SpellID of the highest ranking spell for SpellName
function GetSpellID(SpellName)
  local SpellCount = 0
  local ReturnName
  while (SpellName ~= ReturnName) do
    SpellCount = SpellCount + 1
    ReturnName = GetSpellName(SpellCount, BOOKTYPE_SPELL)
  end
  while (SpellName == ReturnName) do
    SpellID = SpellCount
    SpellCount = SpellCount + 1
    ReturnName = GetSpellName(SpellCount, BOOKTYPE_SPELL)    
    if (SpellName ~= ReturnName) then
      break
    end
  end
  return SpellID
end

-- Value of true is returned if cooldown has passed
function CheckCooldown(SpellID)
  if (GetSpellCooldown(SpellID, 0) == 0) then
    return true
  else
    return false
  end
end





function TextParse(InputString)
   local Text = InputString;
   local TextLength = 1;
   local OutputTable = {};
   local OTIndex = 1;
   local StartAt = 1;
   local StopAt = 1;
   local TextStart = 1;
   local TextStop = 1;
   local TextRemaining = 1;
   local NextSpace = 1;
   local Chunk = "";
   local Iterations = 1;
   local EarlyError = false;

   if ((Text ~= nil) and (Text ~= "")) then
      -- strip out any extra spaces, ie any more than ONE space at a time.
      while (string.find(Text, "  ") ~= nil) do
         Text = string.gsub(Text, "  ", " ");
      end

      -- if text consisted of only spaces
      if (string.len(Text) <= 1) then
         EarlyError = true;
      end

      -- if there is a leading or trailing space, remove.
      if EarlyError ~= true then
        TextStart = 1;
        TextStop = string.len(Text);

        if (string.sub(Text, TextStart, TextStart) == " ") then
           TextStart = TextStart+1;
        end

        if (string.sub(Text, TextStop, TextStop) == " ") then
           TextStop = TextStop-1;
        end

        Text = string.sub(Text, TextStart, TextStop);
      end

      -- break up the string.

      OTIndex = 1;
      TextRemaining = string.len(Text);

      while (StartAt <= TextRemaining) and (EarlyError ~= true) do

         -- NextSpace is the index of the next space in the string.
         NextSpace = string.find(Text, " ",StartAt);
         -- if there isn't another space, then StopAt is the length of the rest of the
         -- string, otherwise it's just before the next space.
         if (NextSpace ~= nil) then
            StopAt = (NextSpace - 1);
         else
            StopAt = string.len(Text);
            LetsEnd = true;
         end

         Chunk = string.sub(Text, StartAt, StopAt);
         OutputTable[OTIndex] = Chunk;
         OTIndex = OTIndex + 1;

         StartAt = StopAt + 2;

      end
   else
      OutputTable[1] = "Error: Bad value passed to TextParse!";
   end

   if (EarlyError ~= true) then
      return OutputTable;
   else
      return {"Error: Bad value passed to TextParse!"};
   end
end






-- Provide a list of the special names for debuffs
function showAllUnitDebuffs(sUnitname) 
  local iIterator = 1
  DEFAULT_CHAT_FRAME:AddMessage(string.format("%s", "Debuff"), 1, 1, 0)
  while (UnitDebuff(sUnitname, iIterator)) do
    DEFAULT_CHAT_FRAME:AddMessage(format(UnitDebuff("target", iIterator)), 1, 1, 0)
    iIterator = iIterator + 1
    DEFAULT_CHAT_FRAME:AddMessage("---", 1, 1, 0)    
  end
end

-- Provide a list of the special names for buffs
function showAllUnitBuffs(sUnitname) 
  local iIterator = 1
  DEFAULT_CHAT_FRAME:AddMessage(format("[%s] Buffs", sUnitname))
  while (UnitBuff(sUnitname, iIterator)) do
    DEFAULT_CHAT_FRAME:AddMessage(UnitBuff(sUnitname, iIterator), 1, 1, 0)    
    iIterator = iIterator + 1
  end
  DEFAULT_CHAT_FRAME:AddMessage("---", 1, 1, 0)    
end
